var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _class, _temp;

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';

import Calendar from './Calendar';

import sizes from '../../utils/sizes';
import { getMoment, getValidDateString, dateSanitizer } from '../../utils/date';

var Datepicker = (_temp = _class = function (_Component) {
  _inherits(Datepicker, _Component);

  function Datepicker(props) {
    _classCallCheck(this, Datepicker);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this.setInputValue = function () {
      var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.props.date;
      return _this.setState({ inputValue: getValidDateString(_extends({}, _this.props, { date: date })) });
    };

    _this.handleClick = function (e) {
      if (_this.isClickOutside(e)) {
        _this.handleCloseAndBlur();
      }
    };

    _this.isClickOutside = function (e) {
      return _this.datepicker && _this.datepicker.compareDocumentPosition(e.target) < 16;
    };

    _this.handleShowCalendar = function () {
      return _this.setState({ isShowCalendar: true }, function () {
        return document.addEventListener('click', _this.handleClick);
      });
    };

    _this.handleCloseAndBlur = function () {
      document.removeEventListener('click', _this.handleClick);
      _this.setState({ isShowCalendar: false });
      _this.handleBlur();
    };

    _this.handleChangeInput = function (e) {
      _this.setState({ inputValue: _this.props.sanitizer(e.target.value) });
      _this.props.onChange(e);
    };

    _this.handleClearInput = function () {
      return _this.setState({ inputValue: '' }, _this.handleBlur);
    };

    _this.handleBlur = function () {
      return _this.props.onBlur({
        target: {
          id: _this.props.id,
          type: _this.props.type,
          name: _this.props.name,
          value: getValidDateString({
            date: _this.state.inputValue,
            inputFormat: _this.props.outputFormat,
            outputFormat: _this.props.inputFormat
          }),
          moment: getMoment(_this.state.inputValue, _this.props.inputFormat)
        }
      });
    };

    _this.state = {
      inputValue: getValidDateString(props),
      isShowCalendar: false
    };
    return _this;
  }

  Datepicker.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (getValidDateString(prevProps) !== getValidDateString(this.props)) {
      this.setInputValue();
    }
  };

  Datepicker.prototype.render = function render() {
    var _this2 = this;

    var _props = this.props,
        id = _props.id,
        name = _props.name,
        type = _props.type,
        dateFromProps = _props.date,
        inputFormat = _props.inputFormat,
        outputFormat = _props.outputFormat,
        placeholder = _props.placeholder,
        disabled = _props.disabled,
        size = _props.size,
        minDate = _props.minDate,
        maxDate = _props.maxDate,
        className = _props.className,
        onChange = _props.onChange,
        onBlur = _props.onBlur,
        sanitizer = _props.sanitizer,
        props = _objectWithoutProperties(_props, ['id', 'name', 'type', 'date', 'inputFormat', 'outputFormat', 'placeholder', 'disabled', 'size', 'minDate', 'maxDate', 'className', 'onChange', 'onBlur', 'sanitizer']);

    var _state = this.state,
        inputValue = _state.inputValue,
        isShowCalendar = _state.isShowCalendar;


    return React.createElement(
      'div',
      {
        className: 'position-relative ' + className,
        ref: function ref(_ref) {
          _this2.datepicker = _ref;
        }
      },
      React.createElement(
        'div',
        { className: 'input-group' },
        React.createElement('input', _extends({}, props, {
          id: id,
          name: name,
          type: type,
          placeholder: placeholder,
          disabled: disabled,
          value: inputValue,
          className: 'form-control form-control-' + size,
          onChange: this.handleChangeInput,
          onFocus: this.handleShowCalendar
        })),
        React.createElement(
          'span',
          { className: 'input-group-append' },
          inputValue ? React.createElement(
            'button',
            {
              disabled: disabled,
              className: 'btn btn-secondary',
              onClick: this.handleClearInput
            },
            React.createElement('i', { className: 'fa fa-fw fa-remove' })
          ) : React.createElement(
            'button',
            {
              disabled: disabled,
              className: 'btn btn-secondary',
              onClick: this.handleShowCalendar
            },
            React.createElement('i', { className: 'fa fa-fw fa-calendar' })
          )
        )
      ),
      React.createElement(
        'div',
        {
          className: 'position-absolute mt-1 ' + (isShowCalendar ? '' : 'd-none'),
          style: { zIndex: 4 }
        },
        React.createElement(Calendar, {
          date: getMoment(inputValue, inputFormat),
          minDate: getMoment(minDate, outputFormat),
          maxDate: getMoment(maxDate, outputFormat),
          onChange: this.setInputValue,
          onClose: this.handleCloseAndBlur
        })
      )
    );
  };

  return Datepicker;
}(Component), _class.defaultProps = {
  id: '',
  name: '',
  type: 'text',
  date: '',
  placeholder: '',
  disabled: false,
  size: 'md',
  inputFormat: 'DD.MM.YYYY',
  outputFormat: 'YYYY-MM-DD',
  minDate: '',
  maxDate: '',
  className: '',
  onChange: function onChange() {},
  onBlur: function onBlur() {},
  sanitizer: dateSanitizer
}, _temp);
Datepicker.propTypes = process.env.NODE_ENV !== "production" ? {
  id: PropTypes.string,
  name: PropTypes.string,
  type: PropTypes.string,
  date: PropTypes.oneOfType([PropTypes.string, PropTypes.instanceOf(moment)]),
  placeholder: PropTypes.string,
  disabled: PropTypes.bool,
  size: PropTypes.oneOf(sizes),
  inputFormat: PropTypes.string,
  outputFormat: PropTypes.string,
  minDate: PropTypes.oneOfType([PropTypes.string, PropTypes.instanceOf(moment)]),
  maxDate: PropTypes.oneOfType([PropTypes.string, PropTypes.instanceOf(moment)]),
  className: PropTypes.string,
  onChange: PropTypes.func,
  onBlur: PropTypes.func,
  sanitizer: PropTypes.func
} : {};


Datepicker.Calendar = Calendar;

export default Datepicker;