'use strict';

exports.__esModule = true;

var _class, _temp, _initialiseProps;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _date = require('../../utils/date');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var WEEKDAYS = Array(7).fill(0).map(function (zero, index) {
  return (0, _moment2.default)().weekday(index).format('dd');
});

var MONTHS = Array(12).fill(0).map(function (zero, index) {
  return (0, _moment2.default)().month(index).format('MMMM');
});

var VIEWTYPE = {
  DAYS: 0,
  MONTHS: 1,
  YEARS: 2
};

var Calendar = (_temp = _class = function (_Component) {
  _inherits(Calendar, _Component);

  function Calendar(props) {
    _classCallCheck(this, Calendar);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _initialiseProps.call(_this);

    _this.state = {
      date: _this.getValidDate(props),
      viewType: VIEWTYPE.DAYS
    };
    return _this;
  }

  Calendar.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    if ((0, _date.getValidDateString)(prevProps) !== (0, _date.getValidDateString)(this.props)) {
      this.setDate(this.props);
    }
  };

  Calendar.prototype.render = function render() {
    var viewType = this.state.viewType;


    return _react2.default.createElement(
      'div',
      {
        className: 'bg-light p-3 border rounded',
        style: { width: 'calc(19.5rem + 2px)' }
        /* padding-left (1rem) + days (2.5rem * 7) + padding-right (1rem) + border */
      },
      this.renderView(viewType)
    );
  };

  return Calendar;
}(_react.Component), _class.defaultProps = {
  date: null,
  minDate: null,
  maxDate: null,
  onChange: function onChange() {},
  onClose: function onClose() {}
}, _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.setDate = function (props) {
    return _this2.setState({
      date: _this2.getValidDate(props)
    });
  };

  this.getValidDate = function (_ref) {
    var date = _ref.date,
        minDate = _ref.minDate,
        maxDate = _ref.maxDate;

    var validDate = (0, _date.isValidMoment)(date) ? date : (0, _moment2.default)();

    if ((0, _date.isValidMoment)(minDate) && validDate.isBefore(minDate.clone().startOf('day'))) {
      return minDate.clone();
    }

    if ((0, _date.isValidMoment)(maxDate) && validDate.isAfter(maxDate.clone().endOf('day'))) {
      return maxDate.clone();
    }

    return validDate.clone();
  };

  this.handleSetYearsView = function () {
    return _this2.setState({
      viewType: VIEWTYPE.YEARS
    });
  };

  this.handleSetMonthsView = function () {
    return _this2.setState({
      viewType: VIEWTYPE.MONTHS
    });
  };

  this.handleSetDaysView = function () {
    return _this2.setState({
      viewType: VIEWTYPE.DAYS
    });
  };

  this.handleNextMonth = function () {
    if (_this2.isAvailableNext()) {
      _this2.setState({
        date: _this2.state.date.clone().add(1, 'months')
      });
    }
  };

  this.handlePrevMonth = function () {
    if (_this2.isAvailablePrev()) {
      _this2.setState({
        date: _this2.state.date.clone().add(-1, 'months')
      });
    }
  };

  this.handleNextYear = function () {
    if (_this2.isAvailableNext()) {
      _this2.setState({
        date: _this2.state.date.clone().add(1, 'years')
      });
    }
  };

  this.handlePrevYear = function () {
    if (_this2.isAvailablePrev()) {
      _this2.setState({
        date: _this2.state.date.clone().add(-1, 'years')
      });
    }
  };

  this.handleSelectYear = function (year) {
    _this2.props.onChange(_this2.state.date.clone().set('year', year));
    _this2.handleSetMonthsView();
  };

  this.handleSelectMonth = function (month) {
    _this2.props.onChange(_this2.state.date.clone().set('month', month));
    _this2.handleSetDaysView();
  };

  this.handleSelectDay = function (day) {
    _this2.props.onChange(day.clone());
    _this2.props.onClose();
  };

  this.isAvailablePrev = function () {
    var minDate = _this2.props.minDate;
    var date = _this2.state.date;


    if (!(0, _date.isValidMoment)(minDate)) return true;

    var startMonthDate = date.clone().startOf('month');
    return minDate.isBefore(startMonthDate);
  };

  this.isAvailableNext = function () {
    var maxDate = _this2.props.maxDate;
    var date = _this2.state.date;


    if (!(0, _date.isValidMoment)(maxDate)) return true;

    var endDateMonth = date.clone().endOf('month');
    return maxDate.isAfter(endDateMonth);
  };

  this.renderYears = function () {
    var _props = _this2.props,
        minDate = _props.minDate,
        maxDate = _props.maxDate;
    var date = _this2.state.date;


    var yearInCenter = date.year();

    var border = {
      min: (0, _date.isValidMoment)(minDate) ? minDate.year() : 1,
      max: (0, _date.isValidMoment)(maxDate) ? maxDate.year() : Infinity
    };

    var yearsCount = 25;
    var firstYear = yearInCenter - Math.floor(yearsCount / 2);

    return _react2.default.createElement(
      'div',
      null,
      _react2.default.createElement(
        'div',
        { className: 'd-flex mb-1' },
        _react2.default.createElement(
          'button',
          {
            className: 'btn btn-light mr-auto',
            disabled: !_this2.isAvailablePrev(),
            onClick: _this2.handlePrevYear
          },
          _react2.default.createElement('i', { className: 'fa fa-chevron-left' })
        ),
        _react2.default.createElement(
          'button',
          {
            className: 'btn btn-light ml-auto',
            disabled: !_this2.isAvailableNext(),
            onClick: _this2.handleNextYear
          },
          _react2.default.createElement('i', { className: 'fa fa-chevron-right' })
        )
      ),
      _react2.default.createElement(
        'div',
        { className: 'd-flex flex-wrap' },
        Array(yearsCount).fill(0).map(function (zero, index) {
          var year = firstYear + index;

          return _react2.default.createElement(
            'button',
            {
              key: year,
              className: '\n                    btn btn-link px-0\n                    ' + (year === yearInCenter ? 'font-weight-bold' : '') + '\n                  ',
              disabled: year < border.min || year > border.max,
              style: { width: '20%' },
              onClick: function onClick() {
                return year >= border.min && year <= border.max && _this2.handleSelectYear(year);
              }
            },
            year
          );
        })
      )
    );
  };

  this.renderMonths = function () {
    var _props2 = _this2.props,
        minDate = _props2.minDate,
        maxDate = _props2.maxDate;
    var date = _this2.state.date;

    var selectedYear = date.year();
    var selectedMonth = date.month();
    var border = {
      min: (0, _date.isValidMoment)(minDate) && minDate.year() === selectedYear ? minDate.month() : 0,
      max: (0, _date.isValidMoment)(maxDate) && maxDate.year() === selectedYear ? maxDate.month() : 11
    };

    return _react2.default.createElement(
      'div',
      { className: 'd-flex flex-wrap' },
      MONTHS.map(function (month, index) {
        return _react2.default.createElement(
          'button',
          {
            key: month,
            className: '\n              btn btn-link text-capitalize px-0\n              ' + (index === selectedMonth ? 'font-weight-bold' : '') + '\n            ',
            style: { width: 100 / 3 + '%' },
            disabled: index < border.min || index > border.max,
            onClick: function onClick() {
              return index >= border.min && index <= border.max && _this2.handleSelectMonth(index);
            }
          },
          month
        );
      })
    );
  };

  this.renderDays = function () {
    var date = _this2.state.date;


    var startMonthWeek = date.clone().startOf('month').startOf('week');
    var endMonthWeek = date.clone().endOf('month').endOf('week');
    var weeksInMonth = (0, _moment2.default)(endMonthWeek - startMonthWeek).weeks() - 1;

    var weeks = Array(weeksInMonth).fill(0).map(function (zero, weekOfMonth) {
      var week = startMonthWeek.clone().add(weekOfMonth, 'weeks');
      return _react2.default.createElement(
        'div',
        { key: week.valueOf(), className: 'd-flex' },
        WEEKDAYS.map(function (weekday, dayOfWeek) {
          return _this2.renderDay(week.clone().add(dayOfWeek, 'days'));
        })
      );
    });

    return _react2.default.createElement(
      'div',
      null,
      _react2.default.createElement(
        'div',
        { className: 'd-flex mb-1' },
        _react2.default.createElement(
          'button',
          {
            className: 'btn btn-light mr-auto',
            disabled: !_this2.isAvailablePrev(),
            onClick: _this2.handlePrevMonth
          },
          _react2.default.createElement('i', { className: 'fa fa-chevron-left' })
        ),
        _react2.default.createElement(
          'button',
          { className: 'btn btn-link', onClick: _this2.handleSetMonthsView },
          _react2.default.createElement(
            'span',
            { className: 'text-capitalize' },
            MONTHS[date.month()]
          )
        ),
        _react2.default.createElement(
          'button',
          { className: 'btn btn-link', onClick: _this2.handleSetYearsView },
          date.year()
        ),
        _react2.default.createElement(
          'button',
          {
            className: 'btn btn-light ml-auto',
            disabled: !_this2.isAvailableNext(),
            onClick: _this2.handleNextMonth
          },
          _react2.default.createElement('i', { className: 'fa fa-chevron-right' })
        )
      ),
      _react2.default.createElement(
        'div',
        { className: 'd-flex my-2' },
        WEEKDAYS.map(function (weekday) {
          return _react2.default.createElement(
            'div',
            { key: weekday, className: 'text-center text-muted', style: { width: 100 / 7 + '%' } },
            _react2.default.createElement(
              'small',
              null,
              weekday
            )
          );
        })
      ),
      weeks
    );
  };

  this.renderDay = function (day) {
    var _props3 = _this2.props,
        selectedDate = _props3.date,
        minDate = _props3.minDate,
        maxDate = _props3.maxDate;
    var date = _this2.state.date;

    var dayValueOf = day.clone().startOf('day').valueOf();
    var displayedMonth = date.month();
    var todayValueOf = (0, _moment2.default)().startOf('day').valueOf();
    var selectedDateValueOf = (0, _date.isValidMoment)(selectedDate) ? selectedDate.clone().startOf('day').valueOf() : null;
    var isAvailable = !((0, _date.isValidMoment)(minDate) && day.isBefore(minDate.clone().startOf('day')) || (0, _date.isValidMoment)(maxDate) && day.isAfter(maxDate.clone().endOf('day')));

    return _react2.default.createElement(
      'button',
      {
        key: dayValueOf,
        className: '\n          btn rounded-circle p-0 border-0\n          ' + (todayValueOf === dayValueOf ? 'font-weight-bold' : '') + '\n          ' + (selectedDateValueOf === dayValueOf ? 'btn-primary' : 'btn-light') + '\n          ' + (displayedMonth !== day.month() && selectedDateValueOf !== dayValueOf ? 'text-muted' : '') + '\n        ',
        style: {
          width: '2.5rem',
          height: '2.5rem',
          lineHeight: '2.5',
          opacity: isAvailable ? 1 : 0.25
        },
        disabled: !isAvailable,
        onClick: function onClick() {
          return isAvailable && _this2.handleSelectDay(day);
        }
      },
      day.date()
    );
  };

  this.renderView = function (type) {
    if (type === VIEWTYPE.YEARS) {
      return _this2.renderYears();
    } else if (type === VIEWTYPE.MONTHS) {
      return _this2.renderMonths();
    }
    return _this2.renderDays();
  };
}, _temp);
Calendar.propTypes = process.env.NODE_ENV !== "production" ? {
  date: _propTypes2.default.instanceOf(_moment2.default),
  minDate: _propTypes2.default.instanceOf(_moment2.default),
  maxDate: _propTypes2.default.instanceOf(_moment2.default),
  onChange: _propTypes2.default.func,
  onClose: _propTypes2.default.func
} : {};
exports.default = Calendar;
module.exports = exports['default'];