'use strict';

exports.__esModule = true;
exports.dateSanitizer = exports.getValidDateString = exports.isValidMoment = exports.getMoment = undefined;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getMoment = exports.getMoment = function getMoment() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  var format = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  return (0, _moment2.default)(date, format, true).isValid() ? (0, _moment2.default)(date, format) : (0, _moment2.default)(date);
};

var isValidMoment = exports.isValidMoment = function isValidMoment(date) {
  return date instanceof _moment2.default && date.isValid();
};

var getValidDateString = exports.getValidDateString = function getValidDateString(_ref) {
  var _ref$date = _ref.date,
      date = _ref$date === undefined ? '' : _ref$date,
      _ref$inputFormat = _ref.inputFormat,
      inputFormat = _ref$inputFormat === undefined ? '' : _ref$inputFormat,
      _ref$outputFormat = _ref.outputFormat,
      outputFormat = _ref$outputFormat === undefined ? '' : _ref$outputFormat,
      _ref$minDate = _ref.minDate,
      minDate = _ref$minDate === undefined ? '' : _ref$minDate,
      _ref$maxDate = _ref.maxDate,
      maxDate = _ref$maxDate === undefined ? '' : _ref$maxDate;

  var dateMoment = getMoment(date, outputFormat);
  var minDateMoment = getMoment(minDate, outputFormat).startOf('day');
  var maxDateMoment = getMoment(maxDate, outputFormat).endOf('day');

  if (!dateMoment.isValid()) {
    return '';
  }

  if (minDateMoment.isValid() && dateMoment.isBefore(minDateMoment)) {
    return minDateMoment.format(inputFormat);
  }

  if (maxDateMoment.isValid() && dateMoment.isAfter(maxDateMoment)) {
    return maxDateMoment.format(inputFormat);
  }

  return dateMoment.format(inputFormat);
};

var dateSanitizer = exports.dateSanitizer = function dateSanitizer() {
  var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

  var dateArr = value.replace(/[^\d.,/-]/g, '').split(/[.,/-]/);

  var day = Number(dateArr[0]) < 31 ? dateArr[0] : 31;
  var month = Number(dateArr[1]) < 12 ? dateArr[1] : 12;
  var year = dateArr[2];

  if (dateArr.length > 2) {
    return day + '.' + month + '.' + year;
  }

  if (dateArr.length > 1) {
    return day + '.' + month;
  }

  return day;
};